@@ -52,7 +52,7 @@ crf1d_context_t* crf1dc_new(int flag, int L, int T)
 {
     int ret = 0;
     crf1d_context_t* ctx = NULL;
-    
+
     ctx = (crf1d_context_t*)calloc(1, sizeof(crf1d_context_t));
     if (ctx != NULL) {
         ctx->flag = flag;
@@ -368,7 +368,7 @@ void crf1dc_marginal_without_beta(crf1d_context_t* ctx)
     fwd = ALPHA_SCORE(ctx, T-1);
     prob = STATE_MEXP(ctx, T-1);
     veccopy(prob, fwd, L);
-    
+
     /*
         Repeat the following computation for t = T-1,T-2, ..., 1.
             1) Compute p(t-1,i,t,j) using p(t,j)
@@ -468,6 +468,7 @@ floatval_t crf1dc_viterbi(crf1d_context_t* ctx, int *labels)
     int i, j, t;
     int *back = NULL;
     floatval_t max_score, score, *cur = NULL;
+    int argmax_score;
     const floatval_t *prev = NULL, *state = NULL, *trans = NULL;
     const int T = ctx->num_items;
     const int L = ctx->num_labels;
@@ -493,7 +494,7 @@ floatval_t crf1dc_viterbi(crf1d_context_t* ctx, int *labels)
         /* Compute the score of (t, j). */
         for (j = 0;j < L;++j) {
             max_score = -FLOAT_MAX;
-
+            argmax_score = -1;
             for (i = 0;i < L;++i) {
                 /* Transit from (t-1, i) to (t, j). */
                 trans = TRANS_SCORE(ctx, i);
@@ -502,10 +503,11 @@ floatval_t crf1dc_viterbi(crf1d_context_t* ctx, int *labels)
                 /* Store this path if it has the maximum score. */
                 if (max_score < score) {
                     max_score = score;
-                    /* Backward link (#t, #j) -> (#t-1, #i). */
-                    back[j] = i;
+                    argmax_score = i;
                 }
             }
+            /* Backward link (#t, #j) -> (#t-1, #i). */
+            if (argmax_score >= 0) back[j] = argmax_score;
             /* Add the state score on (t, j). */
             cur[j] = max_score + state[j];
         }
@@ -514,6 +516,9 @@ floatval_t crf1dc_viterbi(crf1d_context_t* ctx, int *labels)
     /* Find the node (#T, #i) that reaches EOS with the maximum score. */
     max_score = -FLOAT_MAX;
     prev = ALPHA_SCORE(ctx, T-1);
+    /* Set a score for T-1 to be overwritten later. Just in case we don't
+       end up with something beating -FLOAT_MAX. */
+    labels[T-1] = 0;
     for (i = 0;i < L;++i) {
         if (max_score < prev[i]) {
             max_score = prev[i];
@@ -606,7 +611,7 @@ void crf1dc_debug_context(FILE *fp)
         for (y2 = 0;y2 < L;++y2) {
             for (y3 = 0;y3 < L;++y3) {
                 floatval_t logp;
-                
+
                 labels[0] = y1;
                 labels[1] = y2;
                 labels[2] = y3;
@@ -630,7 +635,7 @@ void crf1dc_debug_context(FILE *fp)
         a = ALPHA_SCORE(ctx, 0)[y1];
         b = BETA_SCORE(ctx, 0)[y1];
         c = 1. / ctx->scale_factor[0];
-        
+
         fprintf(fp, "Check for the marginal probability (0,%d)... ", y1);
         check_values(fp, a * b * c, s / norm);
     }
@@ -647,7 +652,7 @@ void crf1dc_debug_context(FILE *fp)
         a = ALPHA_SCORE(ctx, 1)[y2];
         b = BETA_SCORE(ctx, 1)[y2];
         c = 1. / ctx->scale_factor[1];
-        
+
         fprintf(fp, "Check for the marginal probability (1,%d)... ", y2);
         check_values(fp, a * b * c, s / norm);
     }
@@ -664,7 +669,7 @@ void crf1dc_debug_context(FILE *fp)
         a = ALPHA_SCORE(ctx, 2)[y3];
         b = BETA_SCORE(ctx, 2)[y3];
         c = 1. / ctx->scale_factor[2];
-        
+
         fprintf(fp, "Check for the marginal probability (2,%d)... ", y3);
         check_values(fp, a * b * c, s / norm);
     }
